@@ -1,11 +1,13 @@
 __author__ = 'Brian M Anderson'
+
 # Created on 12/31/2020
-import os, pydicom
+import os
+import pydicom
 import numpy as np
 from pydicom.tag import Tag
 import SimpleITK as sitk
 from skimage import draw
-from skimage.measure import label,regionprops,find_contours
+from skimage.measure import label, regionprops, find_contours
 from threading import Thread
 from multiprocessing import cpu_count
 from queue import *
@@ -22,7 +24,7 @@ def contour_worker(A):
         if item is None:
             break
         else:
-            point_maker.make_output(*item)
+            point_maker.make_output(**item)
         q.task_done()
 
 
@@ -45,6 +47,18 @@ def worker_def(A):
             q.task_done()
 
 
+def sort_xy(x, y):
+    x -= np.min(x)
+    y -= np.min(y)
+    x0 = np.mean(x)
+    y0 = np.mean(y)
+    r = np.sqrt((x - x0) ** 2 + (y - y0) ** 2)
+    angles = np.where((y - y0) > 0, np.arccos((x - x0) / r), 2 * np.pi - np.arccos((x - x0) / r))
+    # mask = np.argsort(angles)
+    # mask = np.abs(mask - np.max(mask))
+    return angles
+
+
 class PointOutputMakerClass(object):
     def __init__(self, image_size_rows, image_size_cols, PixelSize, contour_dict, RS):
         self.image_size_rows, self.image_size_cols = image_size_rows, image_size_cols
@@ -52,7 +66,7 @@ class PointOutputMakerClass(object):
         self.contour_dict = contour_dict
         self.RS = RS
 
-    def make_output(self, annotation, i):
+    def make_output(self, annotation, i, dicom_handle):
         self.contour_dict[i] = []
         regions = regionprops(label(annotation))
         for ii in range(len(regions)):
@@ -64,9 +78,31 @@ class PointOutputMakerClass(object):
                 rows.append(data[iii][0])
                 cols.append(data[iii][1])
             temp_image[rows, cols] = 1
-            contours = find_contours(temp_image, 0)
+            contours = find_contours(temp_image, level=0.5, fully_connected='low', positive_orientation='high')
             for contour in contours:
-                self.contour_dict[i].append(contour)
+                contour = np.squeeze(contour)
+                slope = (contour[1:, 1] - contour[:-1, 1]) / (contour[1:, 0] - contour[:-1, 0])
+                slope_index = None
+                out_contour = []
+                for index in range(len(slope)):
+                    if slope[index] != slope_index:
+                        out_contour.append(contour[index])
+                    slope_index = slope[index]
+                # angles = sort_xy(x=np.asarray(out_contour)[:, 0], y=np.asarray(out_contour)[:, 1])
+                contour = [[float(c[1]), float(c[0]), float(i)] for c in out_contour]
+                # k = [dicom_handle.TransformPhysicalPointToIndex(zz) for zz in k]
+                contour = np.asarray([dicom_handle.TransformContinuousIndexToPhysicalPoint(zz) for zz in contour])
+                # distances = np.sum((contour[1:] - contour[:-1])**2, axis=1)**.5
+                # spacing = dicom_handle.GetSpacing()
+                # spacing_min = np.min(dicom_handle.GetSpacing())
+                # distance = 0
+                # out_contour = []
+                # for index in range(len(distances)):
+                #     distance += distances[index]
+                #     if distance > spacing_min:
+                #         out_contour.append(contour[index])
+                #         distance = 0
+                self.contour_dict[i].append(np.asarray(contour))
 
 
 def poly2mask(vertex_row_coords, vertex_col_coords, shape):
@@ -129,10 +165,11 @@ def return_template_dictionary():
 
 
 class DicomReaderWriter:
-    def __init__(self, description='', rewrite_RT_file=False, delete_previous_rois=True, Contour_Names=None, verbose=True,
+    def __init__(self, description='', rewrite_RT_file=False, delete_previous_rois=True, Contour_Names=None,
+                 verbose=True,
                  template_dir=None, arg_max=True, create_new_RT=True, require_all_contours=True, associations={},
                  iteration=0, get_dose_output=False, flip_axes=(False, False, False), index=0,
-                 series_instances_dictionary={}, **kwargs):
+                 series_instances_dictionary={}):
         """
         :param description: string, description information to add to .nii files
         :param rewrite_RT_file: Boolean, should we re-write the RT structure
@@ -146,7 +183,6 @@ class DicomReaderWriter:
         :param get_dose_output: boolean, collect dose information
         :param flip_axes: tuple(3), axis that you want to flip, defaults to (False, False, False)
         :param series_instances_dictionary: dictionary of series instance UIDs of images and RTs
-        :param kwargs:
         """
         self.series_instances_dictionary = series_instances_dictionary
         self.get_dose_output = get_dose_output
@@ -223,6 +259,8 @@ class DicomReaderWriter:
     def __check_if_all_contours_present__(self):
         self.indexes_with_contours = []
         for index in self.series_instances_dictionary:
+            if self.series_instances_dictionary[index]['Image_Path'] is None:
+                continue
             RTs = self.series_instances_dictionary[index]['RTs']
             true_rois = []
             self.rois_in_case = []
@@ -257,6 +295,33 @@ class DicomReaderWriter:
                 if self.all_contours_exist or not self.require_all_contours:
                     self.indexes_with_contours.append(index)  # Add the index that has the contours
 
+    def return_rois(self, print_rois=True):
+        if print_rois:
+            print('The following ROIs were found')
+            for roi in self.all_rois:
+                print(roi)
+        return self.all_rois
+
+    def where_are_RTs(self, ROIName):
+        print('Please move over to using .where_is_ROI(), as this better represents the definition')
+        self.where_is_ROI(ROIName=ROIName)
+
+    def where_is_ROI(self, ROIName):
+        if ROIName.lower() in self.RTs_with_ROI_Names:
+            print('Contours of {} are located:'.format(ROIName.lower()))
+            for path in self.RTs_with_ROI_Names[ROIName.lower()]:
+                print(path)
+        else:
+            print('{} was not found within the set, check spelling or list all rois'.format(ROIName))
+
+    def which_indexes_have_all_rois(self):
+        if self.Contour_Names:
+            print('The following indexes have all ROIs present')
+            for index in self.indexes_with_contours:
+                print('Index {}, located at {}'.format(index, self.series_instances_dictionary[index]['Image_Path']))
+        else:
+            print('You need to first define what ROIs you want, please use .set_contour_names(roi_list)')
+
     def down_folder(self, input_path):
         print('Please move from down_folder() to walk_through_folders()')
         self.walk_through_folders(input_path=input_path)
@@ -280,21 +345,6 @@ class DicomReaderWriter:
         self.__check_if_all_contours_present__()
         return None
 
-    def where_are_RTs(self, ROIName):
-        if ROIName.lower() in self.RTs_with_ROI_Names:
-            print('Contours of {} are located:'.format(ROIName.lower()))
-            for path in self.RTs_with_ROI_Names[ROIName.lower()]:
-                print(path)
-        else:
-            print('{} was not found within the set, check spelling or list all rois'.format(ROIName))
-
-    def return_rois(self, print_rois=True):
-        if print_rois:
-            print('The following ROIs were found')
-            for roi in self.all_rois:
-                print(roi)
-        return self.all_rois
-
     def add_dicom_to_dictionary_from_path(self, PathDicom):
         self.PathDicom = PathDicom
         self.lstFilesDCM = []
@@ -374,8 +424,7 @@ class DicomReaderWriter:
                         self.RTs_in_case[path] = rois_in_structure
                         self.series_instances_dictionary[index]['RTs'].update(temp_dict)
 
-    def write_parallel(self, out_path, excel_file, thread_count=int(cpu_count()*0.9-1)):
-        self.__check_if_all_contours_present__()
+    def write_parallel(self, out_path, excel_file, thread_count=int(cpu_count() * 0.9 - 1)):
         if not os.path.exists(out_path):
             os.makedirs(out_path)
         q = Queue(maxsize=thread_count)
@@ -445,13 +494,13 @@ class DicomReaderWriter:
             t.join()
 
     def get_images_and_mask(self):
-        assert self.index in self.series_instances_dictionary,\
+        assert self.index in self.series_instances_dictionary, \
             'Index is not present in the dictionary! Set it using set_index(index)'
         self.get_images()
         self.get_mask()
 
     def get_images(self):
-        assert self.index in self.series_instances_dictionary,\
+        assert self.index in self.series_instances_dictionary, \
             'Index is not present in the dictionary! Set it using set_index(index)'
         index = self.index
         series_instance_uid = self.series_instances_dictionary[index]['SeriesInstanceUID']
@@ -476,7 +525,7 @@ class DicomReaderWriter:
             self.dicom_handle_uid = series_instance_uid
 
     def get_mask(self):
-        assert self.index in self.series_instances_dictionary,\
+        assert self.index in self.series_instances_dictionary, \
             'Index is not present in the dictionary! Set it using set_index(index)'
         assert self.Contour_Names, 'If you want a mask, you need to set the contour names you are looking ' \
                                    'for, use set_contour_names(list_of_roi_names).\nIf you just want to look at images ' \
@@ -510,8 +559,8 @@ class DicomReaderWriter:
                     for contour_number in range(len(self.RS_struct.ROIContourSequence)):
                         self.structure_references[
                             self.RS_struct.ROIContourSequence[contour_number].ReferencedROINumber] = contour_number
-                    index = self.structure_references[ROIName_Number[ROI_Name]]
-                    mask = self.contours_to_mask(index)
+                    structure_index = self.structure_references[ROIName_Number[ROI_Name]]
+                    mask = self.contours_to_mask(structure_index)
                     self.mask[..., self.Contour_Names.index(true_name) + 1] += mask
                     self.mask[self.mask > 1] = 1
         if self.flip_axes[0]:
@@ -533,14 +582,14 @@ class DicomReaderWriter:
         Contour_data = self.RS_struct.ROIContourSequence[index].ContourSequence
         for i in range(len(Contour_data)):
             as_array = np.asarray(Contour_data[i].ContourData[:])
-            reshaped = np.reshape(as_array, [as_array.shape[0]//3, 3])
+            reshaped = np.reshape(as_array, [as_array.shape[0] // 3, 3])
             matrix_points = np.asarray([self.dicom_handle.TransformPhysicalPointToIndex(reshaped[i])
                                         for i in range(reshaped.shape[0])])
             self.col_val = matrix_points[:, 0]
             self.row_val = matrix_points[:, 1]
             z_vals = matrix_points[:, 2]
             temp_mask = poly2mask(self.row_val, self.col_val, [self.image_size_rows, self.image_size_cols])
-            temp_mask[self.row_val, self.col_val] = 0
+            # temp_mask[self.row_val, self.col_val] = 0
             mask[z_vals[0], temp_mask] += 1
         mask = mask % 2
         return mask
@@ -617,7 +666,7 @@ class DicomReaderWriter:
                 break
 
         prediction_array = np.squeeze(prediction_array)
-        contour_values = np.max(prediction_array, axis=0) # See what the maximum value is across the prediction array
+        contour_values = np.max(prediction_array, axis=0)  # See what the maximum value is across the prediction array
         while len(contour_values.shape) > 1:
             contour_values = np.max(contour_values, axis=0)
         contour_values[0] = 1  # Keep background
@@ -674,9 +723,8 @@ class DicomReaderWriter:
             make_new = 1
             allow_slip_in = True
             if (Name not in current_names and allow_slip_in) or self.delete_previous_rois:
-                self.RS_struct.StructureSetROISequence.insert(0,copy.deepcopy(self.RS_struct.StructureSetROISequence[0]))
-                # self.RS_struct.ROIContourSequence[0].ContourSequence[0].ContourData = []
-                # self.RS_struct.ROIContourSequence[0].ContourSequence[0].NumberofContourPoints = 0
+                self.RS_struct.StructureSetROISequence.insert(0,
+                                                              copy.deepcopy(self.RS_struct.StructureSetROISequence[0]))
             else:
                 print('Prediction ROI {} is already within RT structure'.format(Name))
                 continue
@@ -688,7 +736,8 @@ class DicomReaderWriter:
             self.RS_struct.StructureSetROISequence[self.struct_index].ROIGenerationAlgorithm = 'SEMIAUTOMATIC'
             if make_new == 1:
                 self.RS_struct.RTROIObservationsSequence.insert(0,
-                    copy.deepcopy(self.RS_struct.RTROIObservationsSequence[0]))
+                                                                copy.deepcopy(
+                                                                    self.RS_struct.RTROIObservationsSequence[0]))
                 if 'MaterialID' in self.RS_struct.RTROIObservationsSequence[self.struct_index]:
                     del self.RS_struct.RTROIObservationsSequence[self.struct_index].MaterialID
             self.RS_struct.RTROIObservationsSequence[self.struct_index].ObservationNumber = new_ROINumber
@@ -697,20 +746,20 @@ class DicomReaderWriter:
             self.RS_struct.RTROIObservationsSequence[self.struct_index].RTROIInterpretedType = 'ORGAN'
 
             if make_new == 1:
-                self.RS_struct.ROIContourSequence.insert(0,copy.deepcopy(self.RS_struct.ROIContourSequence[0]))
+                self.RS_struct.ROIContourSequence.insert(0, copy.deepcopy(self.RS_struct.ROIContourSequence[0]))
             self.RS_struct.ROIContourSequence[self.struct_index].ReferencedROINumber = new_ROINumber
             del self.RS_struct.ROIContourSequence[self.struct_index].ContourSequence[1:]
             self.RS_struct.ROIContourSequence[self.struct_index].ROIDisplayColor = temp_color_list[color_int]
             del temp_color_list[color_int]
-            thread_count = int(cpu_count()*0.9-1)
+            thread_count = int(cpu_count() * 0.9 - 1)
             contour_dict = {}
             q = Queue(maxsize=thread_count)
             threads = []
             kwargs = {'image_size_rows': self.image_size_rows, 'image_size_cols': self.image_size_cols,
                       'PixelSize': self.PixelSize, 'contour_dict': contour_dict, 'RS': self.RS_struct}
 
-            A = [q,kwargs]
-            # pointer_class = Point_Output_Maker_Class(**kwargs)
+            A = [q, kwargs]
+            # pointer_class = PointOutputMakerClass(**kwargs)
             for worker in range(thread_count):
                 t = Thread(target=contour_worker, args=(A,))
                 t.start()
@@ -720,17 +769,15 @@ class DicomReaderWriter:
                 image_locations = np.max(annotations, axis=(1, 2))
                 indexes = np.where(image_locations > 0)[0]
                 for index in indexes:
-                    item = [annotations[index, ...], index]
-                    # pointer_class.make_output(*item)
+                    item = {'annotation': annotations[index], 'i': index, 'dicom_handle': self.dicom_handle}
+                    # pointer_class.make_output(**item)
                     q.put(item)
                 for i in range(thread_count):
                     q.put(None)
                 for t in threads:
                     t.join()
                 for i in contour_dict.keys():
-                    for all_contours in contour_dict[i]:
-                        contours = [[int(contour[1]), int(contour[0]), int(i)] for contour in np.squeeze(all_contours)]
-                        points = [self.dicom_handle.TransformIndexToPhysicalPoint(ii) for ii in contours]
+                    for points in contour_dict[i]:
                         output = np.asarray(points).flatten('C')
                         if contour_num > 0:
                             self.RS_struct.ROIContourSequence[self.struct_index].ContourSequence.append(
@@ -745,15 +792,15 @@ class DicomReaderWriter:
                         self.RS_struct.ROIContourSequence[self.struct_index].ContourSequence[
                             contour_num].ContourData = list(output)
                         self.RS_struct.ROIContourSequence[self.struct_index].ContourSequence[
-                            contour_num].NumberofContourPoints = len(output) // 3
+                            contour_num].NumberOfContourPoints = len(output) // 3
                         contour_num += 1
         self.RS_struct.SOPInstanceUID += '.' + str(np.random.randint(999))
         if self.template or self.delete_previous_rois:
-            for i in range(len(self.RS_struct.StructureSetROISequence),len(self.ROI_Names),-1):
+            for i in range(len(self.RS_struct.StructureSetROISequence), len(self.ROI_Names), -1):
                 del self.RS_struct.StructureSetROISequence[-1]
-            for i in range(len(self.RS_struct.RTROIObservationsSequence),len(self.ROI_Names),-1):
+            for i in range(len(self.RS_struct.RTROIObservationsSequence), len(self.ROI_Names), -1):
                 del self.RS_struct.RTROIObservationsSequence[-1]
-            for i in range(len(self.RS_struct.ROIContourSequence),len(self.ROI_Names),-1):
+            for i in range(len(self.RS_struct.ROIContourSequence), len(self.ROI_Names), -1):
                 del self.RS_struct.ROIContourSequence[-1]
             for i in range(len(self.RS_struct.StructureSetROISequence)):
                 self.RS_struct.StructureSetROISequence[i].ROINumber = i + 1
@@ -797,7 +844,8 @@ class DicomReaderWriter:
             self.RS_struct[key]._value[0].RTReferencedStudySequence[0].RTReferencedSeriesSequence[
                 0].ContourImageSequence.append(temp_segment)
         del \
-        self.RS_struct[key]._value[0].RTReferencedStudySequence[0].RTReferencedSeriesSequence[0].ContourImageSequence[0]
+            self.RS_struct[key]._value[0].RTReferencedStudySequence[0].RTReferencedSeriesSequence[
+                0].ContourImageSequence[0]
 
         new_keys = open(self.key_list)
         keys = {}
@@ -826,7 +874,7 @@ class DicomReaderWriter:
                 origin = dose.GetOrigin()
                 direction = dose.GetDirection()
                 scaling_factor = float(reader.GetMetaData("3004|000e"))
-                dose = sitk.GetArrayFromImage(dose)*scaling_factor
+                dose = sitk.GetArrayFromImage(dose) * scaling_factor
                 if output is None:
                     output = dose
                 else:
